import { DynamoDB, S3 } from 'aws-sdk';
import { withCors } from '../../utils/addingCors';

const dynamoDb = new DynamoDB.DocumentClient();
const s3 = new S3();
const tableName = process.env.TABLE_NAME;
const bucketName = process.env.BUCKET_NAME;
const cloudfrontDomain = process.env.CLOUDFRONT_DOMAIN;

export const handler = async (event: any) => {
  const body = JSON.parse(event.body);

  // Extract userId from the claims
  const userId = event.requestContext.authorizer.claims.sub;
  if (!userId) {
    return withCors(403, { error: 'User not authenticated' });
  }

  if (!body.name || !body.phone || !body.email) {
    return withCors(400, { error: 'Missing required fields' });
  }

  const contactId = event.pathParameters?.contactId;

  if (!contactId) {
    return withCors(400, { error: 'Missing required contact ID' });
  }

  try {
    // Fetch the contact to validate ownership
    const contactResult = await dynamoDb
      .get({
        TableName: tableName!,
        Key: {
          userId: userId,
          id: contactId,
        },
      })
      .promise();

    const contact = contactResult.Item;

    if (!contact) {
      return withCors(404, { error: 'Contact not found' });
    }

    // Ensure the authenticated user owns the contact
    if (contact.userId !== userId) {
      return withCors(403, {
        error: 'You do not have permission to update this contact',
      });
    }

    // Update the contact's image if a new image is provided
    let imageUrl = contact.imageUrl;
    if (body.image) {
      const base64Image = body.image.split(',')[1];
      const imageBuffer = Buffer.from(base64Image, 'base64');
      const imageKey = `contacts/${contactId}.jpg`;

      await s3
        .putObject({
          Bucket: bucketName!,
          Key: imageKey,
          Body: imageBuffer,
          ContentType: 'image/jpeg',
          CacheControl: 'max-age=0, must-revalidate',
        })
        .promise();

      imageUrl = `https://${cloudfrontDomain}/${imageKey}?timestamp=${Date.now()}`;
    }

    // Update the contact in DynamoDB
    const updatedContact = {
      ...contact,
      name: body.name,
      phone: body.phone,
      email: body.email,
      isFavorite: body.isFavorite || contact.isFavorite,
      imageUrl,
    };

    await dynamoDb
      .put({
        TableName: tableName!,
        Item: updatedContact,
      })
      .promise();

    return withCors(200, updatedContact);
  } catch (error) {
    console.error(error);
    return withCors(500, { error: 'Could not update contact' });
  }
};
