import * as cdk from 'aws-cdk-lib';
import * as cognito from 'aws-cdk-lib/aws-cognito';
import * as iam from 'aws-cdk-lib/aws-iam';

//CognitoService
export class AuthenticationStack extends cdk.Stack {
  public readonly userPool: cognito.UserPool;
  public readonly userPoolClient: cognito.UserPoolClient;
  public readonly identityPool: cdk.aws_cognito.CfnIdentityPool;
  public readonly authenticatedRole: iam.Role;

  constructor(scope: cdk.App, id: string, props?: cdk.StackProps) {
    super(scope, id, props);

    this.userPool = new cognito.UserPool(this, 'ContactsAppUserPool', {
      selfSignUpEnabled: true,
      signInAliases: { email: true },
      autoVerify: { email: true },
      userPoolName: 'ContactsAppUserPool',
    });
    //This make visible UserPoolId in Cloud Formation Output
    new cdk.CfnOutput(this, 'ContactsAppUserPoolId', {
      value: this.userPool.userPoolId,
    });

    this.userPoolClient = new cognito.UserPoolClient(
      this,
      'ContactsAppUserPoolClient',
      {
        userPoolClientName: 'ContactAppUserPoolClient',
        userPool: this.userPool,
        authFlows: { userPassword: true },
      }
    );

    //This make visible UserPoolClientId in Cloud Formation Output
    new cdk.CfnOutput(this, 'ContactsAppUserPoolClientId', {
      value: this.userPoolClient.userPoolClientId,
    });

    // Identity Pool
    this.identityPool = new cognito.CfnIdentityPool(
      this,
      'ContactsAppIdentityPool',
      {
        allowUnauthenticatedIdentities: false,
        cognitoIdentityProviders: [
          {
            clientId: this.userPoolClient.userPoolClientId,
            providerName: this.userPool.userPoolProviderName,
          },
        ],
      }
    );

    new cdk.CfnOutput(this, 'ContactsAppIdentityPoolId', {
      value: this.identityPool.ref,
    });

    // Authenticated Role with restricted DynamoDB access
    this.authenticatedRole = new iam.Role(
      this,
      'ContactsAppAuthenticatedRole',
      {
        assumedBy: new iam.FederatedPrincipal(
          'cognito-identity.amazonaws.com',
          {
            StringEquals: {
              'cognito-identity.amazonaws.com:aud': this.identityPool.ref,
            },
            'ForAnyValue:StringLike': {
              'cognito-identity.amazonaws.com:amr': 'authenticated',
            },
          },
          'sts:AssumeRoleWithWebIdentity'
        ),
      }
    );

    this.authenticatedRole.addToPolicy(
      new iam.PolicyStatement({
        actions: ['dynamodb:Query', 'dynamodb:GetItem', 'dynamodb:PutItem'],
        resources: ['arn:aws:dynamodb:*:*:table/Contacts'],
        conditions: {
          'ForAllValues:StringEquals': {
            'dynamodb:LeadingKeys': ['${cognito-identity.amazonaws.com:sub}'],
          },
        },
      })
    );

    // Attach Authenticated Role to Identity Pool
    new cognito.CfnIdentityPoolRoleAttachment(
      this,
      'IdentityPoolRoleAttachment',
      {
        identityPoolId: this.identityPool.ref,
        roles: {
          authenticated: this.authenticatedRole.roleArn,
        },
      }
    );

    cdk.Tags.of(this.userPool).add('Service', 'Auth');
  }
}
