import * as cdk from 'aws-cdk-lib';
import * as lambda from 'aws-cdk-lib/aws-lambda';
import * as lambdaNodejs from 'aws-cdk-lib/aws-lambda-nodejs';
import * as dynamodb from 'aws-cdk-lib/aws-dynamodb';
import * as s3 from 'aws-cdk-lib/aws-s3';
import * as cloudfront from 'aws-cdk-lib/aws-cloudfront';
import * as path from 'path';
import { Effect, PolicyStatement } from 'aws-cdk-lib/aws-iam';

interface LambdaStackProps extends cdk.StackProps {
  contactsTable: dynamodb.Table;
  contactsBucket: s3.Bucket;
  cloudfrontImagesDistribution: cloudfront.Distribution;
}

export class LambdaStack extends cdk.Stack {
  public readonly createContactFunction: lambdaNodejs.NodejsFunction;
  public readonly getContactFunction: lambdaNodejs.NodejsFunction;
  public readonly updateContactFunction: lambdaNodejs.NodejsFunction;
  public readonly deleteContactFunction: lambdaNodejs.NodejsFunction;

  constructor(scope: cdk.App, id: string, props: LambdaStackProps) {
    super(scope, id, props);

    const { contactsTable, contactsBucket, cloudfrontImagesDistribution } =
      props;

    //Create Contact Function Lambda
    this.createContactFunction = new lambdaNodejs.NodejsFunction(
      this,
      'CreateContactFunction',
      {
        functionName: 'CreateContactFunction',
        runtime: lambda.Runtime.NODEJS_20_X,
        entry: path.join(__dirname, '../functions/create-contact/index.ts'),
        handler: 'handler',
        environment: {
          TABLE_NAME: contactsTable.tableName,
          BUCKET_NAME: contactsBucket.bucketName,
          CLOUDFRONT_DOMAIN:
            cloudfrontImagesDistribution.distributionDomainName,
        },
      }
    );

    contactsTable.grantWriteData(this.createContactFunction);
    contactsBucket.grantPut(this.createContactFunction);

    this.createContactFunction.addToRolePolicy(
      new PolicyStatement({
        effect: Effect.ALLOW,
        resources: [contactsTable.tableArn],
        actions: [
          'dynamodb:PutItem',
          'dynamodb:GetItem',
          'dynamodb:UpdateItem',
        ],
      })
    );

    //Get Contact Function Lambda
    this.getContactFunction = new lambdaNodejs.NodejsFunction(
      this,
      'GetContactFunction',
      {
        functionName: 'GetContactFunction',
        runtime: lambda.Runtime.NODEJS_20_X,
        entry: path.join(__dirname, '../functions/get-contact/index.ts'),
        handler: 'handler',
        environment: {
          TABLE_NAME: contactsTable.tableName,
          CLOUDFRONT_DOMAIN:
            cloudfrontImagesDistribution.distributionDomainName,
        },
      }
    );

    contactsTable.grantReadData(this.getContactFunction);

    this.getContactFunction.addToRolePolicy(
      new PolicyStatement({
        effect: Effect.ALLOW,
        resources: [contactsTable.tableArn],
        actions: ['dynamodb:Scan', 'dynamodb:GetItem', 'dynamodb:Query'],
      })
    );

    //Update Contact Function Lambda
    this.updateContactFunction = new lambdaNodejs.NodejsFunction(
      this,
      'UpdateContactFunction',
      {
        functionName: 'UpdateContactFunction',
        runtime: lambda.Runtime.NODEJS_20_X,
        entry: path.join(__dirname, '../functions/update-contact/index.ts'),
        handler: 'handler',
        environment: {
          TABLE_NAME: contactsTable.tableName,
          BUCKET_NAME: contactsBucket.bucketName,
          CLOUDFRONT_DOMAIN:
            cloudfrontImagesDistribution.distributionDomainName,
        },
      }
    );

    contactsTable.grantWriteData(this.updateContactFunction);
    contactsBucket.grantPut(this.updateContactFunction);

    this.updateContactFunction.addToRolePolicy(
      new PolicyStatement({
        effect: Effect.ALLOW,
        resources: [contactsTable.tableArn],
        actions: [
          'dynamodb:PutItem',
          'dynamodb:GetItem',
          'dynamodb:UpdateItem',
        ],
      })
    );

    //Delete Contact Function Lambda
    this.deleteContactFunction = new lambdaNodejs.NodejsFunction(
      this,
      'DeleteContactFunction',
      {
        functionName: 'DeleteContactFunction',
        runtime: lambda.Runtime.NODEJS_20_X,
        entry: path.join(__dirname, '../functions/delete-contact/index.ts'),
        handler: 'handler',
        environment: {
          TABLE_NAME: contactsTable.tableName,
          BUCKET_NAME: contactsBucket.bucketName,
          CLOUDFRONT_DOMAIN:
            cloudfrontImagesDistribution.distributionDomainName,
        },
      }
    );

    contactsTable.grantReadWriteData(this.deleteContactFunction);
    contactsBucket.grantDelete(this.deleteContactFunction);

    this.deleteContactFunction.addToRolePolicy(
      new PolicyStatement({
        effect: Effect.ALLOW,
        resources: [contactsTable.tableArn],
        actions: ['dynamodb:DeleteItem', 'dynamodb:GetItem'],
      })
    );

    this.deleteContactFunction.addToRolePolicy(
      new PolicyStatement({
        effect: Effect.ALLOW,
        resources: [`${contactsBucket.bucketArn}/*`],
        actions: ['s3:DeleteObject'],
      })
    );

    cdk.Tags.of(this.createContactFunction).add('Service', 'Lambda');
    cdk.Tags.of(this.getContactFunction).add('Service', 'Lambda');
    cdk.Tags.of(this.updateContactFunction).add('Service', 'Lambda');
    cdk.Tags.of(this.deleteContactFunction).add('Service', 'Lambda');
  }
}
