import * as cdk from 'aws-cdk-lib';
import * as dynamodb from 'aws-cdk-lib/aws-dynamodb';
import * as cloudfront from 'aws-cdk-lib/aws-cloudfront';
import * as s3 from 'aws-cdk-lib/aws-s3';
import * as origins from 'aws-cdk-lib/aws-cloudfront-origins';

export class StorageStack extends cdk.Stack {
  public readonly contactsTable: dynamodb.Table;
  public readonly contactsBucket: s3.Bucket;
  public readonly cloudfrontImagesDistribution: cloudfront.Distribution;

  constructor(scope: cdk.App, id: string, props?: cdk.StackProps) {
    super(scope, id, props);

    //DynamoDB Contacts
    this.contactsTable = new dynamodb.Table(this, 'ContactsAppTable', {
      partitionKey: { name: 'userId', type: dynamodb.AttributeType.STRING },
      sortKey: { name: 'id', type: dynamodb.AttributeType.STRING }, // Optional for storing specific records per user
      tableName: 'Contacts',
      removalPolicy: cdk.RemovalPolicy.DESTROY,
    });

    // Add a GSI for querying by `userId`
    this.contactsTable.addGlobalSecondaryIndex({
      indexName: 'userId-index', // Name of the GSI
      partitionKey: { name: 'userId', type: dynamodb.AttributeType.STRING }, // GSI partition key
      projectionType: dynamodb.ProjectionType.ALL, // Include all attributes in the GSI (can be optimized)
    });

    cdk.Tags.of(this.contactsTable).add('Service', 'Database');

    //Contacts Images S3

    this.contactsBucket = new s3.Bucket(this, 'ContactsAppImagesBucket', {
      bucketName: 'contacts-app.images',
      removalPolicy: cdk.RemovalPolicy.DESTROY,
      autoDeleteObjects: true,
      blockPublicAccess: s3.BlockPublicAccess.BLOCK_ALL,
    });

    // Create CloudFront for Images distribution
    this.cloudfrontImagesDistribution = new cloudfront.Distribution(
      this,
      'CloudfrontImagesDistribution',
      {
        defaultBehavior: {
          origin: origins.S3BucketOrigin.withOriginAccessControl(
            this.contactsBucket
          ),
          viewerProtocolPolicy:
            cloudfront.ViewerProtocolPolicy.REDIRECT_TO_HTTPS, // Redirect HTTP to HTTPS
          cachePolicy: cloudfront.CachePolicy.CACHING_DISABLED, // Adjust Cache Policy
        },
      }
    );

    this.contactsBucket.addToResourcePolicy(
      new cdk.aws_iam.PolicyStatement({
        actions: ['s3:GetObject'],
        resources: [`${this.contactsBucket.bucketArn}/*`],
        principals: [
          new cdk.aws_iam.ServicePrincipal('cloudfront.amazonaws.com'),
        ],
        conditions: {
          StringEquals: {
            'AWS:SourceArn': `arn:aws:cloudfront::${
              cdk.Stack.of(this).account
            }:distribution/${this.cloudfrontImagesDistribution.distributionId}`,
          },
        },
      })
    );

    cdk.Tags.of(this.contactsBucket).add('Service', 'Storage');
    cdk.Tags.of(this.cloudfrontImagesDistribution).add(
      'Service',
      'CloudFrontImagesDistribution'
    );
  }
}
